########################################################
# Compute statistics of the input data and output them
# in a Latex table
########################################################
# (c) Andreas Spitz, spitz@informatik.uni-heidelberg.de
########################################################

library(plyr)
library(xtable)

# names of input files
inFileFeeds <- "data_newscit_feeds.txt"
inFileEdges <- "data_newscit_edges.txt"
inFileNodes <- "data_newscit_nodes.txt"
inFileOutlet <- "data_newscit_outlets.txt"
inFileTemps <- "data_newscit_tempExp.txt"

# date range
minDate <- as.POSIXct("2015-11-01", format = "%Y-%m-%d")
maxDate <- as.POSIXct("2017-10-31", format = "%Y-%m-%d")

# read feeds
# newsOutletID feedTypeID RSSFeedURI Language StartDate CountArticle
colc <- c("character", "character", "character", "character", "character", "integer")
feeds <- read.table(inFileFeeds, sep="\t", header=TRUE, colClasses=colc)

# read edge
# sourceID targetID
colc <- c("integer", "integer")
edges <- read.table(inFileEdges, sep="\t", header=TRUE, colClasses=colc)

# read node data
# articleID publicationDate newsOutletID feedTypeID articleURI
colc <- c("integer", "character", "character", "character", "character")
nodes <- read.table(inFileNodes, sep="\t", header=TRUE, colClasses=colc, quote="", comment="")
nodes$publicationDate <- as.POSIXct(nodes$publicationDate, format = "%Y-%m-%d %H:%M:%S")

#read outlet data
# abbrOutlet newsOutletID startDate fullName
colc <- c("character", "character", "character", "character")
outlets <- read.table(inFileOutlet, sep="\t", header=TRUE, colClasses=colc, quote="", comment="")

#read temporal expression data
colc <- c("integer", "integer","character")
temps <- read.table(inFileTemps, sep="\t", header=TRUE, colClasses=colc, quote="", comment="")

######

# compute number of days active in network
outlets$days <- as.integer(maxDate - as.POSIXct(outlets$startDate, format = "%Y-%m-%d"))
outlets$days <- pmin(outlets$days, as.integer(maxDate-minDate))

# compute avg. number of articles per day
artcounts <- as.data.frame(table(nodes$newsOutletID))
names(artcounts) <- c("newsOutletID", "nArt")
outlets <- merge(outlets,artcounts)
outlets$avgArt <- outlets$nArt / outlets$days

# compute avg. number of temporal expression per article
tempsPlus <- merge(temps,nodes[,c("articleID", "newsOutletID")])
counts <- count(tempsPlus, c("articleID", "newsOutletID"))
avgs <- aggregate(counts$freq, by=list(counts$newsOutletID), FUN=mean)
names(avgs) <- c("newsOutletID","avgTemp")
outlets <- merge(outlets, avgs)

# compute fraction of incoming external edges / outgoing external edges
edgeArt <- nodes[,c("articleID", "newsOutletID")]
names(edgeArt) <- c("targetID","targetOutlet")
edges <- merge(edges,edgeArt)
names(edgeArt) <- c("sourceID","sourceOutlet")
edges <- merge(edges,edgeArt)
edges$match <- as.integer(edges$sourceOutlet == edges$targetOutlet)

# incoming edges
edgesIn <- count(edges, c("targetOutlet"))
names(edgesIn) <- c("newsOutletID", "total")
edgesInType <- count(edges, c("targetOutlet","match"))
names(edgesInType) <- c("newsOutletID", "match", "internal")
edgesIn <- merge(edgesIn, edgesInType[edgesInType$match == 1,c("newsOutletID","internal")])
names(edgesInType) <- c("newsOutletID", "match", "external")
edgesIn <- merge(edgesIn, edgesInType[edgesInType$match == 0,c("newsOutletID","external")])
edgesIn$pIn <- edgesIn$external / edgesIn$total

outlets <- merge(outlets, edgesIn[,c("newsOutletID","pIn")], all.x = TRUE)

# outgoing edges
edgesOut <- count(edges, c("sourceOutlet"))
names(edgesOut) <- c("newsOutletID", "total")
edgesOutType <- count(edges, c("sourceOutlet","match"))
names(edgesOutType) <- c("newsOutletID", "match", "internal")
edgesOut <- merge(edgesOut, edgesOutType[edgesOutType$match == 1,c("newsOutletID","internal")])
names(edgesOutType) <- c("newsOutletID", "match", "external")
edgesOut <- merge(edgesOut, edgesOutType[edgesOutType$match == 0,c("newsOutletID","external")])
edgesOut$pOut <- edgesOut$external / edgesOut$total

outlets <- merge(outlets, edgesOut[,c("newsOutletID","pOut")], all.x = TRUE)

# replace NA values generated by no incoming/outgoing external edges for some outlets
outlets[is.na(outlets)] <- 0

# create result data frame for table generation
result <- outlets[,c("abbrOutlet", "fullName", "days", "avgArt", "avgTemp", "pIn", "pOut")]
result$pIn <- result$pIn * 100 # make percentage
result$pOut <- result$pOut * 100 # make percentage

# create table
cat("Results as a table (Table 1 from the paper)\n\n")
tableRes <- xtable(result, digits=c(0,0,0,0,1,1,1,1))
print(tableRes, include.rownames=FALSE)

# precisions of dates:
cat("\nOverview of date precisions of the temporal expressions\n")
datePrec <- data.frame(prec=c("day","month","year"), length=c(10,7,4))
temps$length <- nchar(temps$tempExp)
tempCounts <- count(temps, c("length"))
datePrec <- merge(datePrec,tempCounts)
datePrec$percent <- 100 * datePrec$freq / sum(datePrec$freq)
print(datePrec)





