########################################################
# Generate the features for date prediction from the
# network.
# NOTE: the output file is already included as a data
# file. This script does not need to be re-run!
########################################################
# (c) Andreas Spitz, spitz@informatik.uni-heidelberg.de
########################################################

library(plyr)
library(xtable)
library(igraph)

# names of input files
inFileFeeds <- "data_newscit_feeds.txt"
inFileEdges <- "data_newscit_edges.txt"
inFileNodes <- "data_newscit_nodes.txt"
inFileOutlet <- "data_newscit_outlets.txt"
inFileTemps <- "data_newscit_tempExp.txt"

# date range
minDate <- as.POSIXct("2015-11-01", format = "%Y-%m-%d")
maxDate <- as.POSIXct("2017-10-31", format = "%Y-%m-%d")

# date range for temporal expressions
tempExpMinDate <- "2000"
tempExpMaxDate <- "2020"

# output file name
outfileData <- "data_newscit_features.txt"

# read feeds
cat("Reading data")
# newsOutletID feedTypeID RSSFeedURI Language StartDate CountArticle
colc <- c("character", "character", "character", "character", "character", "integer")
feeds <- read.table(inFileFeeds, sep="\t", header=TRUE, colClasses=colc)

# read edge
# sourceID targetID
colc <- c("integer", "integer")
edges <- read.table(inFileEdges, sep="\t", header=TRUE, colClasses=colc)

# read node data
# articleID publicationDate newsOutletID feedTypeID articleURI
colc <- c("integer", "character", "character", "character", "character")
nodes <- read.table(inFileNodes, sep="\t", header=TRUE, colClasses=colc, quote="", comment="")
nodes$publicationDate <- as.POSIXct(nodes$publicationDate, format = "%Y-%m-%d %H:%M:%S")

#read outlet data
# abbrOutlet newsOutletID startDate fullName
colc <- c("character", "character", "character", "character")
outlets <- read.table(inFileOutlet, sep="\t", header=TRUE, colClasses=colc, quote="", comment="")

#read temporal expression data
colc <- c("integer", "integer","character")
temps <- read.table(inFileTemps, sep="\t", header=TRUE, colClasses=colc, quote="", comment="")
cat(". Done.\n\n")
######

# generate features
cat("Computing features\n")

# degrees
cat("  Degrees")
g <- graph.data.frame(edges[,c("sourceID","targetID")], directed=TRUE, vertices=nodes[,c("articleID")])
indeg <- degree(g, mode = "in")
outdeg <- degree(g, mode = "out")
degree <- degree(g, mode = "all")

tmp1 <- data.frame(nodeID = as.integer(names(indeg)), degIN = indeg)
tmp2 <- data.frame(nodeID = as.integer(names(outdeg)), degOUT = outdeg)
tmp3 <- data.frame(nodeID = as.integer(names(degree)), degALL = degree)
features <- merge(tmp1,tmp2)
features <- merge(features,tmp3)
cat(". Done.\n")

# local clustering coefficient
cat("  Local cc")
cc <- transitivity(g, type="local")
cc[is.na(cc)] <- 0
tmpCC <- data.frame(nodeID = as.integer(V(g)$name), localCC = cc)
features <- merge(features,tmpCC)
cat(". Done.\n")

# centrality
cat("  PageRank centrality")
pr <- page_rank(g, algo = "prpack")
tmpPR <- data.frame(nodeID = as.integer(names(pr$vector)), centPR = pr$vector)
features <- merge(features,tmpPR)
cat(". Done.\n")

# compute closeness centrality
cat("  Closeness centrality")
closOUT <- closeness(g, mode = "out", normalized = TRUE)
tmpCLOSout <- data.frame(nodeID = as.integer(names(closOUT)), centCLOSout = closOUT)
closIN <- closeness(g, mode = "in", normalized = TRUE)
tmpCLOSin <- data.frame(nodeID = as.integer(names(closIN)), centCLOSin = closIN)
features <- merge(features,tmpCLOSin)
features <- merge(features,tmpCLOSout)
cat(". Done.\n")

# compute betweenness centrality
cat("  Betweenness centrality")
cenBETW <- betweenness(g, directed = TRUE)
tmpBETW <- data.frame(nodeID = as.integer(names(cenBETW)), centBETW = cenBETW)
features <- merge(features,tmpBETW)
cat(". Done.\n")

# add publishing time information to edges
dateNodes <- nodes[,c("articleID", "publicationDate")]
dateNodes$publicationDate <- as.integer(dateNodes$publicationDate)
names(dateNodes) <- c("sourceID", "sourceTime")
dateEdges <- merge(edges,dateNodes)
names(dateNodes) <- c("targetID", "targetTime")
dateEdges <- merge(dateEdges,dateNodes)
names(dateNodes) <- c("nodeID", "Y_pubTime")

# function for computing max time spans
timespan <- function(x) {
    return(max(x) - min(x))
}

# incoming edge publication times
cat("  Incoming time features")
tmp <- data.frame(nodeID = nodes$articleID)
inDateMin <- aggregate(dateEdges$sourceTime,by=list(dateEdges$targetID), FUN=min)
names(inDateMin) <- c("nodeID","inDateMin")
tmp <- merge(tmp,inDateMin,all.x = TRUE)

inDateMax <- aggregate(dateEdges$sourceTime,by=list(dateEdges$targetID), FUN=max)
names(inDateMax) <- c("nodeID","inDateMax")
tmp <- merge(tmp,inDateMax,all.x = TRUE)

inDateMean <- aggregate(dateEdges$sourceTime,by=list(dateEdges$targetID), FUN=mean)
names(inDateMean) <- c("nodeID","inDateMean")
tmp <- merge(tmp,inDateMean,all.x = TRUE)

inDateStdDev <- aggregate(dateEdges$sourceTime,by=list(dateEdges$targetID), FUN=sd)
names(inDateStdDev) <- c("nodeID","inDateStdDev")
tmp <- merge(tmp,inDateStdDev,all.x = TRUE)

inDateSpan <- aggregate(dateEdges$sourceTime,by=list(dateEdges$targetID), FUN=timespan)
names(inDateSpan) <- c("nodeID","inDateSpan")
tmp <- merge(tmp,inDateSpan,all.x = TRUE)
cat(". Done.\n")

# outgoing edge publication times
cat("  Outgoing time features")
tmp <- data.frame(nodeID = nodes$articleID)
outDateMin <- aggregate(dateEdges$targetTime,by=list(dateEdges$sourceID), FUN=min)
names(outDateMin) <- c("nodeID","outDateMin")
tmp <- merge(tmp,outDateMin,all.x = TRUE)

outDateMax <- aggregate(dateEdges$targetTime,by=list(dateEdges$sourceID), FUN=max)
names(outDateMax) <- c("nodeID","outDateMax")
tmp <- merge(tmp,outDateMax,all.x = TRUE)

outDateMean <- aggregate(dateEdges$targetTime,by=list(dateEdges$sourceID), FUN=mean)
names(outDateMean) <- c("nodeID","outDateMean")
tmp <- merge(tmp,outDateMean,all.x = TRUE)

outDateStdDev <- aggregate(dateEdges$targetTime,by=list(dateEdges$sourceID), FUN=sd)
names(outDateStdDev) <- c("nodeID","outDateStdDev")
tmp <- merge(tmp,outDateStdDev,all.x = TRUE)

outDateSpan <- aggregate(dateEdges$targetTime,by=list(dateEdges$sourceID), FUN=timespan)
names(outDateSpan) <- c("nodeID","outDateSpan")
tmp <- merge(tmp,outDateSpan,all.x = TRUE)
cat(". Done.\n")


# distance features between incoming and outgoing times
cat("  Time distance features")
edgesOut <- dateEdges[,c("sourceID","targetID","targetTime")]
names(edgesOut) <- c("nodeID","outID","outTime")
edgesIn <- dateEdges[,c("targetID","sourceID","sourceTime")]
names(edgesIn) <- c("nodeID","inID","inTime")
distEdges <- merge(edgesIn,edgesOut)
distEdges$dist <- distEdges$inTime - distEdges$outTime

tmp <- data.frame(nodeID = nodes$articleID)
distMin <- aggregate(distEdges$dist,by=list(distEdges$nodeID), FUN=min)
names(distMin) <- c("nodeID","distMin")
tmp <- merge(tmp,distMin,all.x = TRUE)

distMax <- aggregate(distEdges$dist,by=list(distEdges$nodeID), FUN=max)
names(distMax) <- c("nodeID","distMax")
tmp <- merge(tmp,distMax,all.x = TRUE)

distMean <- aggregate(distEdges$dist,by=list(distEdges$nodeID), FUN=mean)
names(distMean) <- c("nodeID","distMean")
tmp <- merge(tmp,distMean,all.x = TRUE)

distStdDev <- aggregate(distEdges$dist,by=list(distEdges$nodeID), FUN=sd)
names(distStdDev) <- c("nodeID","distStdDev")
tmp <- merge(tmp,distStdDev,all.x = TRUE)
cat(". Done.\n")


# temporal expression features
cat("  Temporal expression features")
# 1. Padding to middle of month / middle of year for short expressions
# 2. limitation to relevant time span
tt <- temps[,c("articleID", "tempExp")]
charcounts <- nchar(tt$tempExp)
tt$tempExp[charcounts == 4] <- paste(tt$tempExp[charcounts == 4], "-06-30", sep="")
tt$tempExp[charcounts == 7] <- paste(tt$tempExp[charcounts == 7], "-15", sep="")
tt <- tt[tt$tempExp > tempExpMinDate,]
tt <- tt[tt$tempExp < tempExpMaxDate,]
tt$date <- as.POSIXct(tt$tempExp, format = "%Y-%m-%d")
tt <- tt[!is.na(tt$date),] # remove impossible dates (e.g. February 29, 2017)
tt$intDate <- as.integer(tt$date)
tt <- tt[,c("articleID", "intDate")]
# add temporal expression information to edges
names(tt) <- c("sourceID","intDate")
tempEdges <- merge(edges,tt)
# compute features

tmp <- data.frame(nodeID = nodes$articleID)
tempExpMin <- aggregate(tempEdges$intDate, by=list(tempEdges$targetID), FUN=min)
names(tempExpMin) <- c("nodeID","tempExpMin")
tmp <- merge(tmp,tempExpMin,all.x = TRUE)

tempExpMax <- aggregate(tempEdges$intDate, by=list(tempEdges$targetID), FUN=max)
names(tempExpMax) <- c("nodeID","tempExpMax")
tmp <- merge(tmp,tempExpMax,all.x = TRUE)

tempExpMean <- aggregate(tempEdges$intDate, by=list(tempEdges$targetID), FUN=mean)
names(tempExpMean) <- c("nodeID","tempExpMean")
tmp <- merge(tmp,tempExpMean,all.x = TRUE)

tempExpStdDev <- aggregate(tempEdges$intDate, by=list(tempEdges$targetID), FUN=sd)
names(tempExpStdDev) <- c("nodeID","tempExpStdDev")
tmp <- merge(tmp,tempExpStdDev,all.x = TRUE)

tempExpSpan <- aggregate(tempEdges$intDate, by=list(tempEdges$targetID), FUN=timespan)
names(tempExpSpan) <- c("nodeID","tempExpSpan")
tmp <- merge(tmp,tempExpSpan,all.x = TRUE)
cat(". Done.\n")


# add label data Y (publication time)
cat("  Adding label data Y")
features <- merge(features,dateNodes)
cat(". Done.\n")


# save to file
cat("\nWriting data to file")
write.table(features, file = outfileData, append = FALSE, quote = FALSE, sep = " ", row.names = FALSE)
cat(". Done.\n")






