########################################################
# Predict document creation times (DCT) from the features.
########################################################
# (c) Andreas Spitz, spitz@informatik.uni-heidelberg.de
########################################################

#### SETTINGS ALL ####
######################

# Select prediction method
# one of "BASE","LR","BAY","RF","SVM","NN","GB"
predictionMethod <- "GB"

# k-fold validation settings
niter <- 1
nfold <- 10

# random seed
randSeed <- 42

# enable test mode
testMode <- TRUE
testDataSize <- 1000

# use temporal expressions
useTempExp <- TRUE
tempExpNames <- c("tempExpMin", "tempExpMax", "tempExpMean", "tempExpStdDev", "tempExpSpan")

# names of input file
inFileBinary <- "data_newscit_features_imputed.RData"


#### SETTINGS BY METHOD ####
############################

# Neuralnet network settings
nnNodes <- c(14)    # number of nodes in hidden layers, each component corresponds to one layer
nnThreshold <- 1.0  # convergence threshold
nnSteps <- 1e+06    # number of max steps before aborting
nnRep <- 1          # number of repetitions

# SVM settings
svmKernel <- "radial"    # svm kernel
svmEpsilon <- 0.1

# Bayesian regression settings
bayDistribution <- "laplace"

# Gradient boost methods
nGBtrees <- 20000
GBshrinkage <- 0.001
intDepth <- 5
gbDistribution <- "laplace"
gbVerbose <- FALSE


#### PREPARE DATA ####
######################

# names of output files
outFileBinary <- paste("regressionResults_",predictionMethod,"_niter",niter,"_nfold",nfold,"_rseed",randSeed,"_tempExp",useTempExp,".RData",sep="")

# read input data
cat("Reading data")
# Data frames in input file: dataImpSimple dataImpSimpleStand
load(inFileBinary)
cat(". Done.\n")

if (!useTempExp) {
    dataImpSimple <- dataImpSimple[,!(names(dataImpSimple) %in% tempExpNames)]
    dataImpSimpleStand <- dataImpSimpleStand[,!(names(dataImpSimpleStand) %in% tempExpNames)]
}

# set random seed
set.seed(randSeed)

# sample data for test run
if (testMode) {

    #Randomly shuffle the data --- NOTE: assumes nrow(dataSimple) == nrow(dataSimpleStand)
    shuffleIndex <- sample(nrow(dataImpSimple))

    dataImpSimple <- dataImpSimple[shuffleIndex,]
    dataImpSimple <- dataImpSimple[1:testDataSize,]

    dataImpSimpleStand <- dataImpSimpleStand[shuffleIndex,]
    dataImpSimpleStand <- dataImpSimpleStand[1:testDataSize,]

    outFileBinary <- paste("test",testDataSize,"_",outFileBinary,sep="")
}


#### PREPARE DEPENDENT VARIABLES ####
#####################################

# load libraries
if (predictionMethod == "NN") { library(neuralnet) }
if (predictionMethod == "RF") { library(randomForest) }
if (predictionMethod == "SVM") { library(e1071) }
if (predictionMethod == "BAY") { library(bayesreg) }
if (predictionMethod == "GB") { library(gbm) }

# standardize output and store values for restoring values (only for NN)
if (predictionMethod == "NN") {
    st_mean <- mean(dataImpSimpleStand$Y_pubTime)
    st_sd <- sd(dataImpSimpleStand$Y_pubTime)
    dataImpSimpleStand$Y_pubTime <- (dataImpSimpleStand$Y_pubTime - st_mean) / st_sd
}

# result list
list.pred <- list()
list.model <- list()


#### PREDICTION FUNCTIONS ####
##############################

#######################################
# FUNCTION: neural network prediction
predictNN <- function(trainData, testData) {

    trainInput <- trainData[,!(names(trainData) %in% c("nodeID", "bIN", "bOUT"))]
    testInput <- testData[,!(names(testData) %in% c("nodeID", "bIN", "bOUT","Y_pubTime"))]

    # weird formula construction, see
    # https://www.r-bloggers.com/fitting-a-neural-network-in-r-neuralnet-package/
    n <- names(trainInput)
    form <- as.formula(paste("Y_pubTime ~", paste(n[!n %in% "Y_pubTime"], collapse = " + ")))
    nn <- neuralnet(form, data=trainInput, hidden=nnNodes, linear.output=TRUE, threshold = nnThreshold, lifesign = "full", lifesign.step = 10000, rep=nnRep, stepmax = nnSteps)

    nn.pred <- compute(nn,testInput)
    testData$pred <- nn.pred$net.result * st_sd + st_mean
    testData$Y_pubTime <- testData$Y_pubTime * st_sd + st_mean

    resultdf <- data.frame(nodeID=testData$nodeID, indeg=testData$bIN, outdeg=testData$bOUT, alldeg=(testData$bIN & testData$bOUT), pred=testData$pred, Y=testData$Y_pubTime)

    result <- list()
    result[[1]] <- resultdf
    result[[2]] <- nn

    return(result)
}

#######################################
# FUNCTION: random forest prediction
predictRF <- function(trainData, testData) {

    trainInput <- trainData[,!(names(trainData) %in% c("nodeID", "bIN", "bOUT"))]
    testInput <- testData[,!(names(testData) %in% c("nodeID", "bIN", "bOUT"))]

    rf <- randomForest(Y_pubTime ~ ., data=trainInput, importance=TRUE)
    testData$pred <- predict(rf, testInput)

    resultdf <- data.frame(nodeID=testData$nodeID, indeg=testData$bIN, outdeg=testData$bOUT, alldeg=(testData$bIN & testData$bOUT), pred=testData$pred, Y=testData$Y_pubTime)

    result <- list()
    result[[1]] <- resultdf
    result[[2]] <- rf

    return(result)
}

#######################################
# FUNCTION: gradient boost prediction
predictGB <- function(trainData, testData) {

    trainInput <- trainData[,!(names(trainData) %in% c("nodeID", "bIN", "bOUT"))]
    testInput <- testData[,!(names(testData) %in% c("nodeID", "bIN", "bOUT"))]

    gb <- gbm(Y_pubTime ~ ., data=trainInput, distribution = gbDistribution, shrinkage = GBshrinkage, n.trees = nGBtrees, interaction.depth = intDepth, verbose=gbVerbose)
    testData$pred <- predict(gb, testInput, nGBtrees)

    resultdf <- data.frame(nodeID=testData$nodeID, indeg=testData$bIN, outdeg=testData$bOUT, alldeg=(testData$bIN & testData$bOUT), pred=testData$pred, Y=testData$Y_pubTime)

    result <- list()
    result[[1]] <- resultdf
    result[[2]] <- gb

    return(result)
}

#######################################
# FUNCTION: support vector machine prediction
predictSVM <- function(trainData, testData) {

    trainInput <- trainData[,!(names(trainData) %in% c("nodeID", "bIN", "bOUT"))]
    testInput <- testData[,!(names(testData) %in% c("nodeID", "bIN", "bOUT"))]

    svmmodel <- svm(Y_pubTime ~ ., trainInput, kernel=svmKernel, epsilon=svmEpsilon)
    testData$pred <- predict(svmmodel, testInput)

    resultdf <- data.frame(nodeID=testData$nodeID, indeg=testData$bIN, outdeg=testData$bOUT, alldeg=(testData$bIN & testData$bOUT), pred=testData$pred, Y=testData$Y_pubTime)

    result <- list()
    result[[1]] <- resultdf
    result[[2]] <- svmmodel

    return(result)
}

#######################################
# FUNCTION: Bayesian regression prediction
predictBAY <- function(trainData, testData) {

    trainInput <- trainData[,!(names(trainData) %in% c("nodeID", "bIN", "bOUT"))]
    testInput <- testData[,!(names(testData) %in% c("nodeID", "bIN", "bOUT"))]

    bay <- bayesreg(Y_pubTime ~ ., trainInput, model=bayDistribution)
    testData$pred <- predict(bay, testInput)

    resultdf <- data.frame(nodeID=testData$nodeID, indeg=testData$bIN, outdeg=testData$bOUT, alldeg=(testData$bIN & testData$bOUT), pred=testData$pred, Y=testData$Y_pubTime)

    result <- list()
    result[[1]] <- resultdf
    result[[2]] <- bay

    return(result)
}

#######################################
# FUNCTION: Linear regression prediction
predictLR <- function(trainData, testData) {

    trainInput <- trainData[,!(names(trainData) %in% c("nodeID", "bIN", "bOUT"))]
    testInput <- testData[,!(names(testData) %in% c("nodeID", "bIN", "bOUT"))]

    lr <- lm(Y_pubTime ~ ., trainInput)
    testData$pred <- predict(lr, testInput)

    resultdf <- data.frame(nodeID=testData$nodeID, indeg=testData$bIN, outdeg=testData$bOUT, alldeg=(testData$bIN & testData$bOUT), pred=testData$pred, Y=testData$Y_pubTime)

    result <- list()
    result[[1]] <- resultdf
    result[[2]] <- lr

    return(result)
}

#######################################
# FUNCTION: Baseline prediction (mean of incoming and outgoing dates)
predictBASE <- function(trainData, testData) {

    resultdf <- data.frame(nodeID=testData$nodeID, indeg=testData$bIN, outdeg=testData$bOUT, alldeg=(testData$bIN & testData$bOUT), pred=rowMeans(cbind(testData$inDateMean, testData$outDateMean)), Y=testData$Y_pubTime)

    result <- list()
    result[[1]] <- resultdf
    result[[2]] <- NA

    return(result)
}


#### PERFORM EVALUATION ####
############################

for (i in 1:niter) {
    
    #Randomly shuffle the data --- NOTE: assumes nrow(dataSimple) == nrow(dataSimpleStand)
    shuffleIndex <- sample(nrow(dataImpSimple))
    dataImpSimpleStand <- dataImpSimpleStand[shuffleIndex,]
    dataImpSimple <- dataImpSimple[shuffleIndex,]

    # Create 10 equally size folds --- NOTE: assumes nrow(dataSimple) == nrow(dataSimpleStand)
    folds <- cut(seq(1,nrow(dataImpSimple)),breaks=nfold,labels=FALSE)

    # Perform 10 fold cross training
    for(f in 1:nfold) {
        cat(paste(predictionMethod," iteration ",i,", fold ",f,"\n",sep=""))

        testIndexes <- which(folds==f,arr.ind=TRUE)

        testData <- dataImpSimple[testIndexes,]
        trainData <- dataImpSimple[-testIndexes,]

        testDataStand <- dataImpSimpleStand[testIndexes,]
        trainDataStand <- dataImpSimpleStand[-testIndexes,]

        if (predictionMethod == "NN") {
            result <- predictNN(trainDataStand,testDataStand)
        }

        if (predictionMethod == "RF") {
            result <- predictRF(trainData,testData)
        }

        if (predictionMethod == "GB") {
            result <- predictGB(trainData,testData)
        }

        if (predictionMethod == "SVM") {
            result <- predictSVM(trainData,testData)
        }

        if (predictionMethod == "BAY") {
            result <- predictBAY(trainDataStand,testDataStand)
        }

        if (predictionMethod == "LR") {
            result <- predictLR(trainDataStand,testDataStand)
        }

        if (predictionMethod == "BASE") {
            result <- predictBASE(trainData,testData)
        }

        index <- paste(predictionMethod,"_i=",i,",f=",f,sep="")
        list.pred[[index]] <- result[[1]]
        list.model[[index]] <- result[[2]]

    }
    cat("\n")
}

# store run information
if (predictionMethod == "NN") {
    parameterInfo <- data.frame(type=predictionMethod, nnNodes=nnNodes, nnThreshold=nnThreshold, nnSteps=nnSteps, nnRep=nnRep)
}

if (predictionMethod == "RF") {
    parameterInfo <- data.frame(type=predictionMethod)
}

if (predictionMethod == "GB") {
    parameterInfo <- data.frame(type=predictionMethod, nTrees=nGBtrees, shrinkage=GBshrinkage, depth=intDepth, distribution=gbDistribution)
}

if (predictionMethod == "SVM") {
    parameterInfo <- data.frame(type=predictionMethod, svmKernel=svmKernel, svmEpsilon=svmEpsilon)
}

if (predictionMethod == "BAY") {
    parameterInfo <- data.frame(type=predictionMethod, bayDistribution=bayDistribution)
}

if (predictionMethod == "LR") {
    parameterInfo <- data.frame(type=predictionMethod)
}

if (predictionMethod == "BASE") {
    parameterInfo <- data.frame(type=predictionMethod)
}

#save to file
cat("Saving to file\n")
save(list.pred, list.model, parameterInfo, file = outFileBinary, compress = TRUE)
cat("Done\n")

# show some output:
# list.pred[["BAY_i=1,f=1"]]




