########################################################
# Evaluate the document creation time (DCT) prediction
# as a boxplot of the spread of absolute errors.
########################################################
# (c) Andreas Spitz, spitz@informatik.uni-heidelberg.de
########################################################

# libraries
library(ggplot2)

# names of input files
inFileComponents <- c("test1000_regressionResults_","_niter1_nfold10_rseed42_tempExpTRUE",".RData")
#inFileComponents <- c("regressionResults_","_niter1_nfold10_rseed42_tempExpTRUE",".RData")

# method names
mnames <- c("BASE","LR","BAY","NN","RF","GB", "SVM")

# names of output files
outFileData <- paste(inFileComponents[1],"plotdata",inFileComponents[2],".txt",sep="")
outFilePlot <- paste(inFileComponents[1],"plot",inFileComponents[2],".pdf",sep="")

################################
# FUNCTION: read data 
# method: string name of method
createOutput <- function(method) {

    # generate output DF
    result <- data.frame(nodeID=integer(0), method=character(0), absErr=numeric(0), nodeType=character(0))

    # load data
    load(paste(inFileComponents[1],method,inFileComponents[2],inFileComponents[3],sep=""))

    for (index in names(list.pred)) {
        tmp <- list.pred[[index]] 
        tmp$absErr <- abs(tmp$pred - tmp$Y)/(60*60*24) # compute error in days

        # combined result
        result <- rbind(result, data.frame(nodeID=tmp$nodeID, absErr=tmp$absErr, method=rep(method,nrow(tmp)), nodeType=rep("combined",nrow(tmp)) ))

        # both incoming and outgoing edges
        sel <- tmp$alldeg == TRUE
        result <- rbind(result, data.frame(nodeID=tmp$nodeID[sel], absErr=tmp$absErr[sel], method=rep(method,sum(sel)), nodeType=rep("in+out",sum(sel)) ))

        # only incoming edges
        sel <- (tmp$indeg == TRUE) & (tmp$outdeg == FALSE)
        result <- rbind(result, data.frame(nodeID=tmp$nodeID[sel], absErr=tmp$absErr[sel], method=rep(method,sum(sel)), nodeType=rep("in",sum(sel)) ))

        # only outgoing edges
        sel <- (tmp$indeg == FALSE) & (tmp$outdeg == TRUE)
        result <- rbind(result, data.frame(nodeID=tmp$nodeID[sel], absErr=tmp$absErr[sel], method=rep(method,sum(sel)), nodeType=rep("out",sum(sel)) ))
    }
    return(result)
}

# output dataframe
plotdata <- data.frame(nodeID=integer(0), method=character(0), absErr=numeric(0), nodeType=character(0))

# read all data
for (m in mnames) {
    cat(paste("Working on method ",m,sep=""))
    tmpdf <- createOutput(m)
    plotdata <- rbind(plotdata,tmpdf)
    cat(". Done\n")
}

# save data to disk
write.table(plotdata, file = outFileData, quote = FALSE, row.names = FALSE)

# If the data is already generated, read the data file starting from here:
# plotdata <- read.table(outFileData, header=TRUE, sep=" ")

plotdata$method <- factor(plotdata$method, levels=mnames)
plotdata$nodeType <- as.character(plotdata$nodeType)
plotdata$nodeType[plotdata$nodeType == "combined"] <- "all"
plotdata$nodeType <- factor(plotdata$nodeType, levels=c("all", "in", "out", "in+out"))

#colors
cred <- "#d53e4f"
corange <- "#fc8d59"
cyellow <- "#fee08b"
clgreen <- "#e6f598"
cdgreen <- "#99d594"
cblue <- "#3288bd"
cpurple <- "#5b42bd" 
colorvalues <- c(cred, corange, cyellow, clgreen, cdgreen, cblue, cpurple)

# plot results
p <- ggplot(plotdata, aes(x = method, y = absErr, fill = method))
p <- p + stat_boxplot(geom = "errorbar", width = 0.3, position=position_dodge(width=0.85))
p <- p + geom_boxplot(outlier.shape = NA, position=position_dodge(width=0.85), width = 0.7)
p <- p + scale_x_discrete(name = "regression method") + coord_cartesian(ylim = c(5,250))
p <- p + scale_fill_manual(values = colorvalues)
p <- p + scale_y_continuous(name = "absolute error (days)") + theme_bw()
p <- p + facet_wrap(~nodeType, ncol = 1) + theme(legend.position="bottom") + guides(fill = guide_legend(nrow = 1))
p <- p + theme(strip.text.x = element_text(size=12))
p <- p + theme(axis.title.x = element_text(vjust=-0.2, size = 12)) + theme(axis.title.y = element_text(vjust=0.9, size=12))
p <- p + theme(axis.text.x = element_text(color="black"), axis.text.y = element_text(color="black"))
p <- p + theme(plot.margin = unit(c(1, 30, 2, 30), "mm"))
ggsave(outFilePlot, width=12.5, height=18.5, units="cm")



