########################################################
# Evaluate the document creation time (DCT) prediction
# in a table of mean absolute error values.
########################################################
# (c) Andreas Spitz, spitz@informatik.uni-heidelberg.de
########################################################

# libraries
library(ggplot2)

# names of input files
inFileComponents <- c("test1000_regressionResults","_plotdata","_niter1_nfold10_rseed42_tempExpTRUE",".txt")
#inFileComponents <- c("regressionResults","_plotdata","_niter1_nfold10_rseed42_tempExpTRUE",".txt")

# create filenames
inFile <- paste(inFileComponents[1], inFileComponents[2], inFileComponents[3], inFileComponents[4], sep="")

outFilePlot <- paste(inFileComponents[1],"_dayplot",inFileComponents[3],".pdf",sep="")
outFileData <- paste(inFileComponents[1],"_dayplot_plotdata",inFileComponents[3],".txt",sep="")

# method and type names
mnames <- c("BASE","LR","BAY","NN","RF","GB", "SVM")
tnames <- c("combined", "in", "out", "in+out")

# read data
data <- read.table(inFile, header=TRUE, sep=" ")

# prepare plotdata
maxAbs <- ceiling(max(data$absErr))
plotdata <- data.frame(method=character(0), nodeType=character(0), perc=numeric(0), day=integer(0))

# compute overall counts
aggAll <- aggregate(data$absErr, by=list(data$method, data$nodeType), FUN=length)
names(aggAll) <- c("method", "nodeType", "countAll")

for (day in 1:maxAbs) {
    cat(paste("Working on day ",day, " of ", maxAbs,"\n",sep=""))
    dataDev <- data[data$absErr <= (day),]
    aggDev <- aggregate(dataDev$absErr, by=list(dataDev$method, dataDev$nodeType), FUN=length)
    names(aggDev) <- c("method", "nodeType", "countDev")

    combined <- merge(aggAll,aggDev)
    combined$perc <- 100 * combined$countDev / combined$countAll
    combined <- combined[,c("method", "nodeType", "perc")]
    combined$day <- rep(day,nrow(combined))

    plotdata <- rbind(plotdata,combined)
}
cat("\n")

# save data to disk
write.table(plotdata, file = outFileData, quote = FALSE, row.names = FALSE)
# plotdata <- read.table(outFileData, header=TRUE, sep=" ")

plotdata$method <- factor(plotdata$method, levels=mnames)
plotdata$nodeType <- as.character(plotdata$nodeType)
plotdata$nodeType[plotdata$nodeType == "combined"] <- "all"
plotdata$nodeType <- factor(plotdata$nodeType, levels=c("all", "in", "out", "in+out"))

plotdata <- plotdata[plotdata$day <= 60,]

#colors
cred <- "#d53e4f"
corange <- "#fc8d59"
cyellow <- "#fee08b"
clgreen <- "#e6f598"
cdgreen <- "#99d594"
cblue <- "#3288bd"
cpurple <- "#5b42bd"
colorvalues <- c(cred, corange, cyellow, clgreen, cdgreen, cblue, cpurple)
labelvalues <- c("BASE"="BASE  ","LR"="LR  ","BAY"="BAY  ","NN"="NN  ","RF"="RF  ","GB"="GB  ", "SVM"="SVM  ")

# plot results
p <- ggplot(plotdata, aes(x = day, y = perc, colour = method, linetype=method))
p <- p + geom_line(size = 0.8)
p <- p + scale_x_continuous(name = "absolute error (days)")
p <- p + scale_colour_manual(values = colorvalues, labels=labelvalues)
p <- p + scale_linetype_discrete(labels=labelvalues)
p <- p + scale_y_continuous(name = "recall (percentage of predictions < absolute error)")
p <- p + theme_bw()
p <- p + facet_wrap(~nodeType, ncol = 1) + theme(legend.position="bottom") + guides(col = guide_legend(nrow = 1))
p <- p + theme(strip.text.x = element_text(size=12))
p <- p + theme(axis.title.y = element_text(vjust=0.9, size=12))
p <- p + theme(axis.title.x = element_text(vjust=-0.2, size = 12)) 
p <- p + theme(axis.text.x = element_text(color="black"), axis.text.y = element_text(color="black"))
p <- p + theme(plot.margin = unit(c(1, 35, 2, 35), "mm"))
ggsave(outFilePlot, width=13.5, height=18.5, units="cm")



