########################################################
# Extract and plot the importance of features for the
# random forest regressor.
########################################################
# (c) Andreas Spitz, spitz@informatik.uni-heidelberg.de
########################################################

# libraries
library(ggplot2)
library(randomForest)
library(plyr)
library(scales)

# input file name
inFileComponents <- c("test1000_regressionResults_RF_niter1_nfold10_rseed42_tempExpTRUE",".RData")
#inFileComponents <- c("regressionResults_RF_niter1_nfold10_rseed42_tempExpTRUE",".RData")

# names of output files
outFileData <- paste(inFileComponents[1],"_feature_importance_plotdata.txt",sep="")
outFilePlot <- paste(inFileComponents[1],"_feature_importance_plot.pdf",sep="")

# feature types
typeTempExp <- c("tempExpMax", "tempExpMean", "tempExpMin", "tempExpSpan", "tempExpStdDev")
typeTopology <- c("centBETW", "centCLOSin", "centCLOSout", "centPR", "degALL", "degIN", "degOUT", "localCC")
typeDate <- c("distMax", "distMean", "distMin", "distStdDev", "inDateMax", "inDateMean", "inDateMin", "inDateSpan", "inDateStdDev", "outDateMax", "outDateMean", "outDateMin", "outDateSpan", "outDateStdDev")

# labels for features displayed in plot 
featLabels <- c("tempExpMax"=expression("max"~(X["in"])), "tempExpMean"=expression(mu~(X["in"])), "tempExpMin"=expression("min"~(X["in"])), "tempExpSpan"=expression("span"~(X["in"])), "tempExpStdDev"=expression(sigma~(X["in"])), "centBETW"=expression(c[btw]), "centCLOSin"=expression(c["cl,in"]), "centCLOSout"=expression(c["cl,out"]), "centPR"=expression(c[pr]), "degALL"=expression(deg[all]), "degIN"=expression(deg["in"]), "degOUT"=expression(deg[out]), "localCC"=expression(cc), "distMax"=expression("max"~("Dist")), "distMean"=expression(mu~("Dist")), "distMin"=expression("min"~("Dist")), "distStdDev"=expression(sigma~("Dist")), "inDateMax"=expression("max"~(T["in"])), "inDateMean"=expression(mu~(T["in"])), "inDateMin"=expression("min"~(T["in"])), "inDateSpan"=expression("span"~(T["in"])), "inDateStdDev"=expression(sigma~(T["in"])), "outDateMax"=expression("max"~(T["out"])), "outDateMean"=expression(mu~(T["out"])), "outDateMin"=expression("min"~(T["out"])), "outDateSpan"=expression("span"~(T["out"])), "outDateStdDev"=expression(sigma~(T["out"])) )

# load data
load(paste(inFileComponents[1],inFileComponents[2],sep=""))

# output dataframe
data <- data.frame(feature=character(0), importance=numeric(0), iter=integer(0))
nIter <- 0

for (index in names(list.model)) {
    nIter <- nIter + 1
    model <- list.model[[index]]

    imp <- importance(model)
    impDF <- data.frame(feature=row.names(imp), incPurity=imp[,2], iter=rep(nIter,nrow(imp)))
    data <- rbind(data,impDF)
}

plotdata <- ddply(data, c("feature"), summarise,
                  N    = length(incPurity),
                  mean = mean(incPurity),
                  sd   = sd(incPurity),
                  se   = sd / sqrt(N)
)

# save data to disk
write.table(plotdata, file = outFileData, quote = FALSE, row.names = FALSE)
# plotdata <- read.table(outFileData, header=TRUE, sep=" ")

maxMean <- max(plotdata$mean)
plotdata$mean <- plotdata$mean / maxMean
plotdata$sd <- plotdata$sd / maxMean

facLevels <- plotdata$feature[order(-plotdata$mean)]
plotdata$feature <- factor(plotdata$feature, levels=facLevels)
plotdata$wrap <- rep("Feature importance: random forest",nrow(plotdata))
plotdata$type[plotdata$feature %in% typeTempExp] <- "tempExp"
plotdata$type[plotdata$feature %in% typeTopology] <- "topology"
plotdata$type[plotdata$feature %in% typeDate] <- "date"

corange <- "#fc8d59"
cyellow <- "#fee08b"
cdgreen <- "#99d594"
cpurple <- "#5b42bd"

fillColors <- c("date" = cpurple, "tempExp" = cdgreen, "topology" = cyellow)

p <- ggplot(plotdata, aes(x=feature, y=mean, fill=type, shape=type))
p <- p + geom_errorbar(aes(ymin=mean+sd, ymax=mean-sd), colour="black", width=.4)
p <- p + geom_point(size=2.3, colour="black")
p <- p + facet_wrap(~wrap, ncol = 1)
p <- p + scale_fill_manual(values=fillColors)
p <- p + scale_x_discrete("", labels = featLabels, expand=c(0,0.8)) 
p <- p + scale_y_log10(breaks = c(0.00001, 0.0001, 0.001,0.01,0.1,1), labels = trans_format("log10", math_format(10^.x)))
p <- p + annotation_logticks(sides="l", short = unit(.5,"mm"), mid = unit(1.3,"mm"), long = unit(2.0,"mm"))
p <- p + scale_shape_manual(values=c(21,22,23))
p <- p + theme_bw() + theme(plot.margin = unit(c(1, 1, 1, 1), "mm")) + theme(panel.grid.minor = element_blank())
p <- p + theme(axis.text.x = element_text(color="black"), axis.text.y = element_text(color="black"))
p <- p + ylab("relative importance")  + theme(legend.position="none")
p <- p + theme(strip.text.x = element_text(size=12)) + theme(strip.text.y = element_text(size=12))
p <- p + theme(axis.title.x = element_text(vjust=-0.2, size = 12)) + theme(axis.title.y = element_text(size=12))
p <- p + theme(axis.text.x = element_text(angle=60,hjust=0.95,vjust=0.5,margin=unit(c(-5.5, 0, 0, 0), "mm")))
ggsave(outFilePlot, width=5.8, height=2.2)








